package com.community.huiju.controller;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.internal.util.AlipaySignature;
import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.huiju.common.wxpay.WXPayConstants;
import com.community.huiju.common.wxpay.WXPayUtil;
import com.community.huiju.feign.IFuShiServer;
import com.community.huiju.service.AliMPUserServiceI;
import com.community.huiju.service.WxMPUserServiceI;
import com.google.common.collect.Maps;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import static com.alipay.api.AlipayConstants.CHARSET_UTF8;

/**
 * @author FXF
 * @date 2019-07-08
 */
@RestController
@RefreshScope
@RequestMapping("/")
@Api(value = "支付宝生活号API", description = "支付宝生活号API")
public class AliMPUserController {
	
	private Logger log = LoggerFactory.getLogger(AliMPUserController.class);
	
	@Value("${ali.mp.configs.appId}")
	private String appId;
	
	@Value("${ali.mp.configs.secret}")
	private String secret;
	
	@Value("${alipay.ALIPAY_PUBLIC_KEY}")
	private String ALIPAY_PUBLIC_KEY;
	
	@Autowired
	private AliMPUserServiceI aliMPUserService;
	
	@Autowired
	private IFuShiServer fuShiServer;

	@RequestMapping(value = "/alimp/info", method = RequestMethod.GET)
	@ApiOperation(value = "获取支付宝生活号相关信息", notes = "获取支付宝生活号相关信息")
	@ApiImplicitParams({})
	public ResponseBean getAliMPinfo(){
		ResponseBean responseBean = new ResponseBean();
		Map<String,Object> map = Maps.newHashMap();
		map.put("appId",appId);
		responseBean.addSuccess(map);
		return responseBean;
	}
	
	@RequestMapping(value = "/{communityId}/alimp", method = RequestMethod.GET)
	@ApiOperation(value = "根据CODE获取OPENID并判断是否第一次来,并返回数据", notes = "根据CODE获取OPENID并判断是否第一次来,并返回数据")
	@ApiImplicitParams({@ApiImplicitParam(paramType = "path", dataType = "integer", name = "communityId", value = "小区Id"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "code", value = "支付宝code")})
	public ResponseBean getUseridByCode(@PathVariable Integer communityId, @RequestParam String code){
		ResponseBean responseBean = aliMPUserService.getTemporaryData(code,appId,secret,communityId);
		return responseBean;
	}
	
	
	@RequestMapping(value = "/{communityId}/alimp/records", method = RequestMethod.GET)
	@ApiOperation(value = "根据openid获取缴费记录", notes = "根据openid获取缴费记录")
	@ApiImplicitParams({@ApiImplicitParam(paramType = "path", dataType = "integer", name = "communityId", value = "小区Id"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "opneid", value = "支付宝opneid")})
	public ResponseBean getRecordsByOpenid(@PathVariable Integer communityId, @RequestParam String opneid){
		ResponseBean responseBean = aliMPUserService.getRecordsByOpenid(opneid,communityId);
		return responseBean;
	}
	
	@RequestMapping(value = "/{communityId}/alimp/car", method = RequestMethod.GET)
	@ApiOperation(value = "根据车牌获取数据", notes = "根据车牌获取数据")
	@ApiImplicitParams({@ApiImplicitParam(paramType = "path", dataType = "integer", name = "communityId", value = "小区Id"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "carNo", value = "车牌号")})
	public ResponseBean getParkInfoByCarNo(@PathVariable Integer communityId, @RequestParam String carNo){
		ResponseBean responseBean = new ResponseBean();
		String parkInfoByCarNo = fuShiServer.getParkInfoByCarNo(carNo);
		JSONObject carNoJson = JSONObject.parseObject(parkInfoByCarNo);
		//车辆不在停车场
		if (carNoJson.getInteger("code") == 1){
			responseBean.addError(carNoJson.getString("message"));
			return responseBean;
		}
		
		if (carNoJson.getInteger("code") == 0 ){
			if (carNoJson.getJSONObject("data").getInteger("DataType") != 3){
				responseBean.addSuccess("储值车不支持在线缴费，月卡车请使用银城慧家APP延期续费");
				return responseBean;
			}
			//返回查询的金额
			JSONObject jsonParam = carNoJson.getJSONObject("data").getJSONObject("JsonParam");
			Map<String,Object> resultMap = Maps.newHashMap();
			resultMap.put("ActualAmount",jsonParam.get("ActualAmount"));
			responseBean.addSuccess(resultMap);
		}
		return responseBean;
	}
	
	@ApiOperation(value = "临时车统一下单", notes = "临时车统一下单")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "path", dataType = "integer", name = "communityId", value = "小区Id"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "carNo", value = "车牌号"),
			@ApiImplicitParam(paramType = "query", dataType = "String", name = "openid", value = "支付宝openid")
	})
	@RequestMapping(value = "/{communityId}/alimp/order",method = RequestMethod.GET)
	public ResponseBean aliTmpOrder(@PathVariable Integer communityId,
	                               @RequestParam("carNo") String carNo,
	                               @RequestParam("openid") String openid){
		ResponseBean responseBean = new ResponseBean();
		
		//String parkInfoByCarNo = fuShiServer.getParkInfoByCarNo(carNo);
		//JSONObject carNoJson = JSONObject.parseObject(parkInfoByCarNo);
		//String actualAmount = String.valueOf(Double.valueOf(((BigDecimal) carNoJson.getJSONObject("JsonParam").get("ActualAmount")).toString()) * 100);
		//actualAmount = actualAmount.substring(0, actualAmount.lastIndexOf("."));
		//String resp = wxMPUserService.wxTmpOrder(carNo,actualAmount,communityId,openid,(BigDecimal) carNoJson.getJSONObject("JsonParam").get("ActualAmount"));
		String resp = aliMPUserService.aliTmpOrder(carNo,"1",communityId,openid,new BigDecimal("0.1"));
		
		responseBean.addSuccess(resp);
		return responseBean;
	}
	
	@ApiOperation(value = "手机订单支付完成后回调临时车缴费", notes = "手机订单支付完成后回调临时车缴费")
	@RequestMapping(value = "/aliTmpNotify",method = RequestMethod.POST)
	public void wxBillNotify(HttpServletRequest request, HttpServletResponse response){
		log.info("支付宝 alipay result  start =======");
		Map<String,String> params = new HashMap<String,String>();
		Map requestParams = request.getParameterMap();
		for (Iterator iter = requestParams.keySet().iterator(); iter.hasNext();) {
			String name = (String) iter.next();
			String[] values = (String[]) requestParams.get(name);
			String valueStr = "";
			for (int i = 0; i < values.length; i++) {
				valueStr = (i == values.length - 1) ? valueStr + values[i]
						: valueStr + values[i] + ",";
			}
			//乱码解决，这段代码在出现乱码时使用。
			//valueStr = new String(valueStr.getBytes("ISO-8859-1"), "utf-8");
			params.put(name, valueStr);
		}
		//切记alipaypublickey是支付宝的公钥，请去open.alipay.com对应应用下查看。
		try {
			boolean flag = AlipaySignature.rsaCheckV1(params, ALIPAY_PUBLIC_KEY, CHARSET_UTF8,"RSA2");
			if (flag){
				aliMPUserService.updateTmpOrder(params.get("out_trade_no"));
			}
		} catch (AlipayApiException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
}
