package com.community.huiju.controller;

import com.alibaba.fastjson.JSONObject;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.dao.TpTransactionMapper;
import com.community.huiju.dao.TpUserMapper;
import com.community.huiju.model.TpTransaction;
import com.community.huiju.model.TpUser;
import com.community.huiju.service.SocialServiceI;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpSession;

@RestController
@RefreshScope
@RequestMapping("/")
@Api(value = "物业端的帖子和工单", description = "物业端的帖子和工单")
public class TpTicketAndTransactionController {
    @Autowired
    private SocialServiceI socialServiceI;

    @Autowired
    private TpUserMapper tpUserMapper;

    @ApiOperation(value = "app端话题管理", notes = "app端话题管理")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path",dataType = "String",name = "loginName",value = "电话号码"),
            @ApiImplicitParam(paramType = "query",dataType = "Integer",name = "status",value = "1:被举报话题,2:未读话题,3:已读话题"),
            @ApiImplicitParam(paramType = "query",dataType = "String",name = "transactionTitle",value = "话题搜索"),
            @ApiImplicitParam(paramType = "query",dataType = "Integer",name = "communityId",value = "小区"),
            @ApiImplicitParam(paramType = "query",dataType = "Integer",name = "pageNum",value = "第几页"),
            @ApiImplicitParam(paramType = "query",dataType = "Integer",name = "pageSize",value = "一页多少数据"),
    })
    @RequestMapping(value = "/transactionsManageList/{loginName}",method = RequestMethod.GET)
    public ResponseBean tpTransactionManageList(@PathVariable(value = "loginName") String loginName,
                                   @RequestParam(value = "status") Integer status,
                                   @RequestParam(value = "transactionTitle", required = false) String transactionTitle,
                                   @RequestParam(value = "communityId", required = false) Integer communityId,
                                   @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                   @RequestParam(value = "pageSize", defaultValue = "9999") Integer pageSize) {
       ResponseBean tpTransactionManageList= socialServiceI.tpTransactionManageList(loginName,status,pageNum,pageSize,transactionTitle,communityId);
       return tpTransactionManageList;
    }

    @ApiOperation(value = "app端话题管理批量作废话题",nickname ="app端话题管理批量作废话题")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "paramets", value ="id:帖子id(多个),loginName:电话号码,communityId:小区")})
    @RequestMapping(value="/transactions/delete",method = RequestMethod.POST)
    public ResponseBean transactionsDelete(@RequestBody String paramets){
        ResponseBean  transactionsUpdata= socialServiceI.transactionsDelete(paramets);
        return transactionsUpdata;
    }

    @ApiOperation(value = "app端话题管理清除不实举报",nickname ="app端话题管理清除不实举报")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "paramets", value ="id:帖子id(多个),loginName:电话号码,communityId:小区")})
    @RequestMapping(value="/tpTransactionReport/delete",method = RequestMethod.POST)
    public ResponseBean tpTransactionReportDelete(@RequestBody String paramets){
        ResponseBean  transactionsUpdata= socialServiceI.tpTransactionReportDelete(paramets);
        return transactionsUpdata;
    }

    @ApiOperation(value = "app端话题管理添加话题",nickname ="app端话题管理添加话题")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "paramets", value ="loginName:电话,communityId:小区ID，transactionTitle:话题标题，" +
                    "type:0 代表 二手（卖二手物品）  1 是代表 求购 （买二手物品） 2 是代表 租赁（租二手物品）3其他话题，contentImg：图片，sort:权重值,status:'0 是已作废 1 是已发布  2 是草稿 ")})
    @RequestMapping(value="/transactions/add",method = RequestMethod.POST)
    public ResponseBean transactionsAdd(@RequestBody String paramets){
        ResponseBean  addAnnouncement= socialServiceI.transactionsAdd(paramets);
        return addAnnouncement;
    }

    @ApiOperation(value = "app端话题管理批量已读话题",nickname ="app端话题管理批量已读话题")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "paramets", value ="id:帖子id(多个),loginName:电话号码,communityId:小区")})
    @RequestMapping(value="/transactions/read",method = RequestMethod.POST)
    public ResponseBean transactionsRead(@RequestBody String paramets){
        ResponseBean  transactionsUpdata= socialServiceI.transactionsRead(paramets);
        return transactionsUpdata;
    }

    @ApiOperation(value = "app端工单管理列表", notes = "app端工单管理列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path",dataType = "String",name = "loginName",value = "电话号码"),
            @ApiImplicitParam(paramType = "query",dataType = "Integer",name = "status",value = "0:待分配,3正在处理，9:其他状态"),
            @ApiImplicitParam(paramType = "query",dataType = "Integer",name = "communityId",value = "小区"),
            @ApiImplicitParam(paramType = "query",dataType = "Integer",name = "pageNum",value = "第几页"),
            @ApiImplicitParam(paramType = "query",dataType = "Integer",name = "pageSize",value = "一页多少数据"),
    })
    @RequestMapping(value = "/tpTicketManageList/{loginName}",method = RequestMethod.GET)
    public ResponseBean tpTicketManageList(@PathVariable(value = "loginName") String loginName,
                                                @RequestParam(value = "status") Integer status,
                                                @RequestParam(value = "communityId") Integer communityId,
                                                @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                                                @RequestParam(value = "pageSize", defaultValue = "9999") Integer pageSize) {
        ResponseBean tpTicketManageList= socialServiceI.tpTicketManageList(loginName,status,pageNum,pageSize,communityId);
        return tpTicketManageList;
    }

    @ApiOperation(value = "app端工单管理详情", notes = "app端工单管理列表详情")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path",dataType = "String",name = "tickedID",value = "工单ID"),
            @ApiImplicitParam(paramType = "query",dataType = "String",name = "loginName",value = "电话号码"),
            @ApiImplicitParam(paramType = "query",dataType = "Integer",name = "communityId",value = "小区")
    })
    @RequestMapping(value = "/tpTicketManage/{tickedID}",method = RequestMethod.GET)
    public ResponseBean tpTicketManageDetails(@PathVariable(value = "tickedID") Integer tickedID,
                                              @RequestParam(value = "communityId") Integer communityId,
                                              @RequestParam(value = "loginName") String loginName) {
        ResponseBean tpTicketManageDetails= socialServiceI.tpTicketManageDetails(tickedID,communityId,loginName);
        return tpTicketManageDetails;
    }

    @ApiOperation(value = "app端工单管理回复业主", notes = "app端工单管理回复业主")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "parameter",value = "loginName:手机号,communityId:小区,content:对话内容，imageUrl:[]图片,ticketId:工单id，" +
                    "status:工单状态（0:待分配  1:拒绝受理 2:待处理 3:正在处理 4:待评价 5:已评价 6:已终止）")
    })
    @RequestMapping(value = "/tpTicketManage/reply",method = RequestMethod.POST)
    public ResponseBean tpTicketManageReply(@RequestBody String parameter) {
        ResponseBean tpTicketManageReply= socialServiceI.tpTicketManageReply(parameter);
        return tpTicketManageReply;
    }

    @ApiOperation(value = "app端工单管理受理分配", notes = "app端工单管理受理分配")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "parameter",value = "loginName:手机号,communityId:小区,ticketId:工单id，" +
                    "id:当前处理人id")
    })
    @RequestMapping(value = "/ticketManage/addRecord", method = RequestMethod.POST)
    public ResponseBean ticketManageRecord(@RequestBody String parameter){
        ResponseBean  responseBean = socialServiceI.ticketManageRecord(parameter);
        return responseBean;
    }

    @ApiOperation(value = "app端工单管理拒绝处理", notes = "app端工单管理拒绝处理")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "parameter",value = "loginName:手机号,communityId:小区,ticketId:工单id,textContent:拒绝的内容" )
    })
    @RequestMapping(value = "/updateTicketStatus", method = RequestMethod.POST)
    public ResponseBean updateTicketStatus(@RequestBody String parameter, HttpSession session){
        ResponseBean  responseBean = socialServiceI.updateTicketStatus(parameter);
        return responseBean;
    }

    @ApiOperation(value = "app端工单管理处理完成", notes = "app端工单管理处理完成")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "parameter",value = "loginName:手机号,communityId:小区,content:对话内容，imageUrl:[]图片,ticketId:工单id" )
    })
    @RequestMapping(value = "/updateTicket", method = RequestMethod.POST)
    public ResponseBean updateTicket(@RequestBody String parameter){
        ResponseBean  responseBean = socialServiceI.updateTicket(parameter);
        return responseBean;
    }

    @ApiOperation(value = "app端话题管理话题详情", notes = "app端话题管理话题详情")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", dataType = "integer", name = "communityId", value = "小区Id"),
            @ApiImplicitParam(paramType = "query", dataType = "integer", name = "activityId", value = "帖子id"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "loginName", value = "手机号"),
    })
    @RequestMapping(value = "/getTransactionManage/{communityId}", method = RequestMethod.GET)
    public ResponseBean findUsedDetails(@PathVariable(value = "communityId") Integer communityId, @RequestParam("activityId") Integer activityId,@RequestParam("loginName") String loginName) {
        ResponseBean responseBean = new ResponseBean();
        TpUser tpUser= tpUserMapper.selectLoginName(loginName,communityId);
        TpTransaction activity = socialServiceI.findUsedDetails(activityId, communityId, tpUser.getId());
        responseBean.addSuccess(activity);
        return responseBean;
    }

    @ApiOperation(value = "app端个人中心话题管和工单管理是否展示", notes = "app端个人中心话题管和工单管理是否展示")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", dataType = "integer", name = "communityId", value = "小区Id"),
            @ApiImplicitParam(paramType = "query", dataType = "String", name = "loginName", value = "手机号"),
    })
    @RequestMapping(value = "/accessAuthority/{communityId}", method = RequestMethod.GET)
    public ResponseBean findUsedDetails(@PathVariable(value = "communityId") Integer communityId,@RequestParam("loginName") String loginName) {
        ResponseBean responseBean = new ResponseBean();
        TpUser tpUser= tpUserMapper.selectLoginName(loginName,communityId);
         if (null == tpUser){
             responseBean.addError("此号码无权限");
             return responseBean;
         }
             responseBean.addSuccess("成功");
        return responseBean;
    }

}
