package com.community.huiju.feign;

import com.community.commom.mode.ResponseBean;
import com.community.huiju.config.FeignUploadConfig;
import com.community.huiju.feign.impl.HKServerFallbackImpl;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.multipart.MultipartFile;

@FeignClient(value = "hardware-server", fallback = HKServerFallbackImpl.class, configuration = FeignUploadConfig.class)
public interface IHKServer {

    /**
     * 添加人员
     *
     *  {
     * 	"type": "厂商",
     * 	"person": {
     * 		"openapi_ip_port_http": "服务器地址",
     * 		"appkey": "服务器appkey",
     * 		"secret": "服务器secret",
     * 		"opUserUuid": "操作人员UUID",
     * 		"personNo": "人员编号",
     * 		"personName": "人员姓名",
     * 		"remark": "手机号",
     * 		"deptUuid": "部门uuid"
     *   }
     * }
     *
     *
     * @param jsonString
     * @return
     */
    @PostMapping(value = "/pushPerson", consumes = MediaType.APPLICATION_JSON_UTF8_VALUE)
    String pushPerson(@RequestBody String jsonString);

    /**
     * 修改人员
     *
     *{
     * 	"type": "厂商",
     * 	"person": {
     * 		"openapi_ip_port_http": "服务器地址",
     * 		"appkey": "服务器appkey",
     * 		"secret": "服务器secret",
     * 		"opUserUuid": "操作人员UUID",
     * 		"personNo": "人员编号",
     * 		"personName": "人员姓名",
     * 		"remark": "手机号",
     * 		"deptUuid": "部门uuid",
     * 		"personId": "人员海康ID"
     *        }
     * }
     *
     * @param jsonString
     * @return
     */
    @PostMapping(value = "/updatePerson", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    String updatePerson(@RequestBody String jsonString);

    /**
     * 添加人脸
     *
     *      *  json里面的设备下发字段，没有就不写入
     *      *  比如：不下发门禁人脸权限，就写入 entranceGuardFace 这个字段的json字符串
     *
     * {
     * 	"type": "厂商",
     * 	"face": {
     * 		"doorPermissions": { // 下发门口机权限(可视对讲)\n" +
     * 			"authName": "权限组",
     * 			"personIds": [人员海康ID...],
     * 			"secret": "服务器secret",
     * 			"appkey": "服务器appkey",
     * 			"openapi_ip_port_http\": \"服务器地址\",\n" +
     * 			"opUserUuid\": \"服务器操作人员UUID\",\n" +
     * 			"longNums\": \"门口机(可视对讲)编号\"\n" +
     *                },
     * 		"doorFace": { // 下发门口机人脸权限\n" +
     * 			"secret": "服务器secret",
     * 			"appkey": "服务器appkey",
     * 			"openapi_ip_port_http": "服务器地址",
     * 			"opUserUuid": "服务器操作人员UUID",
     * 			"opType": 1, // 表示下发"
     * 			"personIds": "人员海康ID",
     * 			"longNums": "门口机(可视对讲)编号"
     *        },
     * 		"entranceGuardPermissions": { // 下发门禁权限
     * 			"personIds": "人员海康ID",
     * 			"doorUuids": "门禁点UUID",
     * 			"secret": "服务器secret",
     * 			"appkey": "服务器appkey",
     * 			"openapi_ip_port_http": "服务器操地址",
     * 			"opUserUuid": "服务器操作人员UUID"
     *        },
     * 		"entranceGuardFace": { // 下发门禁人脸权限
     * 			"secret": "服务器secret",
     * 			"appkey": "服务器appkey",
     * 			"openapi_ip_port_http": "服务器操地址",
     * 			"opUserUuid": "服务器操作人员UUID\",\n" +
     * 				"personIds\": \"人员海康ID\",\n" +
     * 				"deviceUuids\": \"门禁设备UUID\",\n" +
     * 				"type": "1"
     *        },
     * 		"person": {
     * 			"secret": "服务器secret",
     * 			"appkey": "服务器appkey",
     * 			"openapi_ip_port_http": "服务器操地址",
     * 			"opUserUuid": "服务器操作人员UUID",
     * 			"personId": "人员海康ID",
     * 			"personNo": "人员海康编号",
     * 			"personName": "人员名称",
     * 			//"facePhoto":"人脸图片",
     * 			"cardNo": "卡号"
     *        }
     *     }
     * }
     *
     * @param faceImg 文件流
     * @param jsonString json字符串
     * @return
     */
    @PostMapping(value = "/addFace", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    String addFace(@RequestPart MultipartFile faceImg, @RequestParam("jsonString") String jsonString);


    /**
     * 修改人脸
     *
     *  json里面的设备下发字段，没有就不写入
     *  比如：不下发门禁人脸权限，就写入 entranceGuardFace 这个字段的json字符串
     *
     *{
     * 	"type": "厂商",
     * 	"face": {
     * 		"doorFace": { // 下发门口机人脸权限\n" +
     * 			"secret": "服务器secret",
     * 			"appkey": "服务器appkey",
     * 			"openapi_ip_port_http": "服务器地址",
     * 			"opUserUuid": "服务器操作人员UUID",
     * 			"opType": 1, // 表示下发"
     * 			"personIds": "人员海康ID",
     * 			"longNums": "门口机(可视对讲)编号"
     *        },
     * 		"entranceGuardFace": { // 下发门禁人脸权限
     * 			"secret": "服务器secret",
     * 			"appkey": "服务器appkey",
     * 			"openapi_ip_port_http": "服务器操地址",
     * 			"opUserUuid": "服务器操作人员UUID\",\n"
     * 			"personIds\": \"人员海康ID\"," +
     * 			"deviceUuids\": \"门禁设备UUID\",
     * 			"type": "1"
     *        },
     * 		"person": {
     * 			"secret": "服务器secret",
     * 			"appkey": "服务器appkey",
     * 			"openapi_ip_port_http": "服务器操地址",
     * 			"opUserUuid": "服务器操作人员UUID",
     * 			"personId": "人员海康ID",
     * 			"personNo": "人员海康编号",
     * 			"personName": "人员名称",
     * 			//"facePhoto":"人脸图片",
     * 			"cardNo": "卡号"
     *        }* 	}
     * }
     *
     * @param faceImg 文件流
     * @param jsonString json字符串
     * @return
     */
    @PostMapping(value = "/updateFace", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    String updateFace(@RequestPart MultipartFile faceImg, @RequestParam("jsonString") String jsonString);


}
