package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.domain.AlipayTradeAppPayModel;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.response.AlipayTradeAppPayResponse;
import com.beust.jcommander.internal.Maps;
import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.huiju.common.perproties.PayNotifyPerproties;
import com.community.huiju.common.wxmp.WxMPUtil;
import com.community.huiju.common.wxpay.QaddWxConfig;
import com.community.huiju.common.wxpay.WXPay;
import com.community.huiju.common.wxpay.WXPayConstants;
import com.community.huiju.common.wxpay.WXPayUtil;
import com.community.huiju.dao.TaTemporaryLicenseOrderMapper;
import com.community.huiju.service.AliMPUserServiceI;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.Instant;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import static com.alipay.api.AlipayConstants.CHARSET_UTF8;

/**
 * @author FXF
 * @date 2019-07-08
 */
@Service("aliMPUserService")
@Slf4j
public class AliMPUserServiceImpl implements AliMPUserServiceI {
	
	@Autowired
	private TaTemporaryLicenseOrderMapper taTemporaryLicenseOrderMapper;
	
	@Autowired
	private PayNotifyPerproties payNotifyPerproties;
	
	@Value("${alipay.APP_PRIVATE_KEY}")
	private String APP_PRIVATE_KEY;
	
	@Value("${alipay.APP_ID}")
	private String ALI_APP_ID;
	
	@Value("${alipay.ALIPAY_PUBLIC_KEY}")
	private String ALIPAY_PUBLIC_KEY;
	
	/**
	 * 获取缴过费用的临时车牌
	 * @param code
	 * @param appId
	 * @param communityId
	 * @return
	 */
	@Override
	public ResponseBean getTemporaryData(String code, String appId, String secret,Integer communityId) {
		ResponseBean responseBean = new ResponseBean();
		Map<String,Object> resultMap = Maps.newHashMap();
		String wxresult = WxMPUtil.getWxMPUserInfo(code,appId,secret);
		JSONObject wxresultJson = JSONObject.parseObject(wxresult);
		String openid = wxresultJson.getString("openid");
		//根据openid查询临时车表查看是否存在关联过得车牌 缴费方式 1是支付宝 2是微信支付
		List<Map<String,Object>> licensePlateList = taTemporaryLicenseOrderMapper.getLicensePlateByOpenid(openid,communityId,"1");
		resultMap.put("openid",openid);
		resultMap.put("licensePlateList",licensePlateList);
		responseBean.addSuccess(resultMap);
		return responseBean;
	}
	
	/**
	 * 根据openid获取缴费记录
	 * @param openid
	 * @param communityId
	 * @return
	 */
	@Override
	public ResponseBean getRecordsByOpenid(String openid, Integer communityId) {
		ResponseBean responseBean = new ResponseBean();
		//根据openid查询临时车表缴费数据 缴费方式 1是支付宝 2是微信支付
		List<Map<String,Object>> licensePlateList = taTemporaryLicenseOrderMapper.getLicensePlateRecordsByOpenid(openid,communityId, "1");
		responseBean.addSuccess(licensePlateList);
		return responseBean;
	}
	
	/**
	 * 临时车下单
	 *
	 *
	 * @param carNo
	 * @param actualAmount
	 * @return
	 */
	@Override
	public String aliTmpOrder(String carNo, String actualAmount, Integer communityId, String openid, BigDecimal price) {
		Random random = new Random();
		String orderNo = String.valueOf(System.currentTimeMillis()) + random.nextInt();
		//下单
		AlipayClient alipayClient = new DefaultAlipayClient("https://openapi.alipay.com/gateway.do", ALI_APP_ID, APP_PRIVATE_KEY, "json", CHARSET_UTF8, ALIPAY_PUBLIC_KEY, "RSA2");
		//实例化具体API对应的request类,类名称和接口名称对应,当前调用接口名称：alipay.trade.app.pay
		AlipayTradeAppPayRequest request = new AlipayTradeAppPayRequest();
		//SDK已经封装掉了公共参数，这里只需要传入业务参数。以下方法为sdk的model入参方式(model和biz_content同时存在的情况下取biz_content)。
		AlipayTradeAppPayModel model = new AlipayTradeAppPayModel();
		model.setBody("临时车缴费");
		model.setSubject("临时车缴费");
		model.setOutTradeNo(orderNo);
		model.setTimeoutExpress("30m");
		model.setTotalAmount(price.toString());
		model.setProductCode("QUICK_MSECURITY_PAY");
		request.setBizModel(model);
		request.setNotifyUrl(payNotifyPerproties.getAliTmpNotify());
		try {
			//这里和普通的接口调用不同，使用的是sdkExecute
			AlipayTradeAppPayResponse response = alipayClient.sdkExecute(request);
			//就是orderString 可以直接给客户端请求，无需再做处理。
			System.out.println(response.getBody());
			//插入缴费信息到临时车标 缴费方式 1是支付宝 2是微信支付
			taTemporaryLicenseOrderMapper.saveTmpOrder(communityId,orderNo,actualAmount,openid,carNo,price,"1");
			return response.getBody();
		} catch (AlipayApiException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	/**
	 * 更新订单状态
	 *
	 * @param outTradeNo
	 * @return
	 */
	@Override
	public String updateTmpOrder(String outTradeNo) {
		//缴费方式 1是支付宝 2是微信支付
		int size = taTemporaryLicenseOrderMapper.updateTmpOrder(outTradeNo, "1");
		if (size > 0 ){
			return "success";
		}
		return "fail";
	}
	
	/**
	 * 根据订单号获取交易信息
	 *
	 * @param outTradeNo
	 * @return
	 */
	@Override
	public Map<String, Object> selectByOrderNumber(String outTradeNo) {
		//缴费方式 1是支付宝 2是微信支付
		return taTemporaryLicenseOrderMapper.selectByOrderNumber(outTradeNo, "1");
	}
	
}
