package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.beust.jcommander.internal.Maps;
import com.community.commom.mode.ResponseBean;
import com.community.huiju.common.perproties.PayNotifyPerproties;
import com.community.huiju.common.wxmp.WxMPUtil;
import com.community.huiju.common.wxpay.HfWxConfig;
import com.community.huiju.common.wxpay.QaddWxConfig;
import com.community.huiju.common.wxpay.WXPay;
import com.community.huiju.common.wxpay.WXPayConstants;
import com.community.huiju.common.wxpay.WXPayUtil;
import com.community.huiju.dao.TaTemporaryLicenseOrderMapper;
import com.community.huiju.service.WxMPUserServiceI;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.Instant;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

/**
 * @author FXF
 * @date 2019-07-08
 */
@Service("wxMPUserService")
@Slf4j
public class WxMPUserServiceImpl implements WxMPUserServiceI {
	
	@Autowired
	private TaTemporaryLicenseOrderMapper taTemporaryLicenseOrderMapper;
	
	@Autowired
	private PayNotifyPerproties payNotifyPerproties;
	
	/**
	 * 获取缴过费用的临时车牌
	 * @param code
	 * @param appId
	 * @param communityId
	 * @return
	 */
	@Override
	public ResponseBean getTemporaryData(String code, String appId, String secret,Integer communityId) {
		ResponseBean responseBean = new ResponseBean();
		Map<String,Object> resultMap = Maps.newHashMap();
		String wxresult = WxMPUtil.getWxMPUserInfo(code,appId,secret);
		JSONObject wxresultJson = JSONObject.parseObject(wxresult);
		String openid = wxresultJson.getString("openid");
		//根据openid查询临时车表查看是否存在关联过得车牌   缴费方式 1是支付宝 2是微信支付
		List<Map<String,Object>> licensePlateList = taTemporaryLicenseOrderMapper.getLicensePlateByOpenid(openid,communityId,"2");
		resultMap.put("openid",openid);
		resultMap.put("licensePlateList",licensePlateList);
		responseBean.addSuccess(resultMap);
		return responseBean;
	}
	
	/**
	 * 根据openid获取缴费记录
	 * @param openid
	 * @param communityId
	 * @return
	 */
	@Override
	public ResponseBean getRecordsByOpenid(String openid, Integer communityId) {
		ResponseBean responseBean = new ResponseBean();
		//根据openid查询临时车表缴费数据 缴费方式 1是支付宝 2是微信支付
		List<Map<String,Object>> licensePlateList = taTemporaryLicenseOrderMapper.getLicensePlateRecordsByOpenid(openid,communityId,"2");
		responseBean.addSuccess(licensePlateList);
		return responseBean;
	}
	
	/**
	 * 临时车下单
	 *
	 *
	 * @param carNo
	 * @param actualAmount
	 * @return
	 */
	@Override
	public Map<String, String> wxTmpOrder(String carNo, String actualAmount, Integer communityId, String openid, BigDecimal price) throws Exception {
		QaddWxConfig config = new QaddWxConfig();
		WXPay wxpay = new WXPay(config);
		Random random = new Random();
		String orderNo = String.valueOf(System.currentTimeMillis()) + random.nextInt();
		//下单
		Map<String, String> data = new HashMap<String, String>();
		data.put("body", "临时车缴费");
		//商品号唯一
		data.put("out_trade_no", orderNo);
		data.put("device_info", "");
		data.put("fee_type", "CNY");
		//金额 后台计算
		data.put("total_fee", actualAmount);
		//终端IP
		data.put("spbill_create_ip", "123.12.12.123");
		//回调地址
		data.put("notify_url", payNotifyPerproties.getWxTmpNotify());
		// 此处指定为APP支付
		data.put("trade_type", "APP");
		Map<String, String> resp = wxpay.unifiedOrder(data);
		log.info("下单成功：{}", resp);
		if (resp.get("result_code").equalsIgnoreCase(WXPayConstants.FAIL)){
			throw new Exception(resp.get("err_code_des"));
		}
		//插入缴费信息到临时车标 缴费方式 1是支付宝 2是微信支付
		taTemporaryLicenseOrderMapper.saveTmpOrder(communityId,orderNo,actualAmount,openid,carNo,price,"2");
		
		//第二次重新生成签名
		Map<String, String> secondSignData = new HashMap<String, String>();
		long timestamp = Instant.now().getEpochSecond();
		secondSignData.put("appid", config.getAppID());
		secondSignData.put("partnerid", config.getMchID());
		secondSignData.put("prepayid", resp.get("prepay_id"));
		secondSignData.put("noncestr", resp.get("nonce_str"));
		secondSignData.put("timestamp", String.valueOf(timestamp));
		secondSignData.put("package", "Sign=WXPay");
		secondSignData.put("sign", WXPayUtil.generateSignature(secondSignData, config.getKey(), WXPayConstants.SignType.MD5));
		
		log.info("订单: {}，订单类型: {}, 状态变更为正在支付", orderNo, carNo);
		return secondSignData;
	}
	
	/**
	 * 更新订单状态
	 *
	 * @param outTradeNo
	 * @return
	 */
	@Override
	public String updateTmpOrder(String outTradeNo) {
		//缴费方式 1是支付宝 2是微信支付
		int size = taTemporaryLicenseOrderMapper.updateTmpOrder(outTradeNo, "2");
		if (size > 0 ){
			return "success";
		}
		return "fail";
	}
	
	/**
	 * 根据订单号获取交易信息
	 *
	 * @param outTradeNo
	 * @return
	 */
	@Override
	public Map<String, Object> selectByOrderNumber(String outTradeNo) {
		//缴费方式 1是支付宝 2是微信支付
		return taTemporaryLicenseOrderMapper.selectByOrderNumber(outTradeNo,  "2");
	}
	
}
