package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.beust.jcommander.internal.Lists;
import com.beust.jcommander.internal.Maps;
import com.community.commom.hk.HKOpenApi;
import com.community.commom.mode.ResponseBean;
import com.community.commom.uuid.IdGen;
import com.community.huiju.common.messagePush;
import com.community.huiju.dao.*;
import com.community.huiju.exception.WisdomException;
import com.community.huiju.model.*;
import com.community.huiju.service.IHKService;
import com.community.huiju.service.userDataHKsiteI;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import sun.misc.BASE64Encoder;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;


@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class userDataHKsiteImpl  implements userDataHKsiteI {

    @Autowired
    private TaUserVerifyMapper taUserVerifyMapper;

    @Autowired
    private TaUserMapper taUserMapper;

    @Autowired
    private TpUnitHkSettingMapper tpUnitHkSettingMapper;


    @Autowired
    private TpEquipmentTreeMapper tpEquipmentTreeMapper;

    private IdGen idGen = IdGen.get();

    @Autowired
    private TaUserHkMapper taUserHkMapper;

    @Autowired
    private TpEquipmentMapper tpEquipmentMapper;


    @Autowired
    private TpConfigurationMapper tpConfigurationMapper;

    @Autowired
    private TaFaceMapper taFaceMapper;

    @Autowired
    private TpMessageMapper tpMessageMapper;

    @Autowired
    private IHKService ihkService;

    @Override
    public ResponseBean cleanHKdata(Integer communityId) {
        ResponseBean responseBean = new ResponseBean();
        //根据小区id查询审核数据
        List<TaUserVerify> userVerifyList = taUserVerifyMapper.selectByCommunityId(communityId);

        int i = 1;
        for (TaUserVerify taUserVerify : userVerifyList){
            log.info("当前编号：{}, 人员审核id：{}", i, taUserVerify.getId());
            //重新构建海康和我们的关系
            TaUserHk taUserHk = taUserHkMapper.selectByUserVerifyIdAndCommunityId(taUserVerify.getId(), taUserVerify.getCommunityId());
            if (null == taUserHk) {
                buildHKData(taUserVerify);
            }
        }

        return responseBean;
    }

    private void buildHKData(TaUserVerify taUserVerify) {

        TaUserHk taUserHk = new TaUserHk();
        taUserHk.setHkPersonNo(idGen.nextId());

        //查询人脸信息
        TaFace taFace = taFaceMapper.getByUserVerifyId(taUserVerify.getId());

        TaUser taUser = taUserMapper.selectByPrimaryKey(taUserVerify.getUserId());

        // 海康权限信息
        TpUnitHkSetting unitHkSetting = getUnitHkSetting(taUserVerify.getCommunityId(), taUserVerify.getUnitId());
        TpEquipmentTree tpEquipmentTree = tpEquipmentTreeMapper.selectByCommunityId(taUserVerify.getCommunityId());
        // 开卡 卡号
        long cardNo = System.currentTimeMillis();
        // 添加卡片
        Map<String,Object> addCardsMap = new HashMap<>();
        addCardsMap.put("secret", tpEquipmentTree.getSecret());
        addCardsMap.put("appkey", tpEquipmentTree.getAppkey());
        addCardsMap.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
        addCardsMap.put("startCardNo", cardNo);
        addCardsMap.put("endCardNo", cardNo);
        addCardsMap.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
        String addCardsJSON = HKOpenApi.addCards(addCardsMap);
        JSONObject addCardsJsonObject = JSONObject.parseObject(addCardsJSON);
        Integer addCardsResultCode = addCardsJsonObject.getInteger("errorCode");
        if (0 != addCardsResultCode) {
            throw new RuntimeException(addCardsJsonObject.getString("errorMessage"));
        }



        // =================== 添加人员 ===========================
        Map<String, Object> parUser = Maps.newHashMap();
        parUser.put("personNo", taUserHk.getHkPersonNo());
        parUser.put("personName", taUser.getUserName());
        // parUser.put("phoneNo", user.getLoginName());
        parUser.put("remark", taUser.getLoginName());
        parUser.put("deptUuid", unitHkSetting.getDepartmentId());
        parUser.put("secret", tpEquipmentTree.getSecret());
        parUser.put("appkey", tpEquipmentTree.getAppkey());
        parUser.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
        parUser.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
        if (null != taFace){
            try {
                parUser.put("facePhoto", encodeImageToBase64(new URL(taFace.getFaceImg())));
            } catch (Exception e) {
                log.info("解析图片出错");
                e.printStackTrace();
            }
            parUser.put("cardNo", cardNo + "");
        }

        Map<String, Object> resultMap = JSONObject.parseObject(HKOpenApi.addUser(parUser), HashMap.class);


        Integer errorCode = (Integer) resultMap.get("errorCode");
        if (errorCode != 0) {
            throw new RuntimeException(String.valueOf(resultMap.get("errorMessage")));
        }
        Map<String, Object> resultDataMap = (Map<String, Object>) resultMap.get("data");
        // 海康人员ID
        Integer personId = (Integer) resultDataMap.get("personId");
        taUserHk.setHkUserId(personId);

        // =================== 添加卡片 =====================



        taUserHk.setHkCardNo(cardNo + "");

        taUserHk.setCommunityId(taUserVerify.getCommunityId());
        taUserHk.setTaUserId(taUserVerify.getId());
        taUserHk.setCreateDate(new Date());
        taUserHk.setCreateUser(1);
        taUserHkMapper.insertSelective(taUserHk);
        // ============== 人员绑定卡片 ======================

        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        LocalDate today = LocalDate.now();
        int year = today.getYear();
        // 加上20, 表示加上20年,默认
        year += 20;

        int month = today.getMonthValue();
        int day = today.getDayOfMonth();

        Long time = null;
        try {
            time = formatter.parse(year + "-" + month + "-" + day).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }

        if (null == taFace){
            // 开卡
            Map<String, Object> parOpenCard = Maps.newHashMap();
            parOpenCard.put("personId", taUserHk.getHkUserId());
            parOpenCard.put("cardNo", Long.valueOf(cardNo));
            parOpenCard.put("startTime", System.currentTimeMillis());
            parOpenCard.put("endTime", time);
            parOpenCard.put("secret", tpEquipmentTree.getSecret());
            parOpenCard.put("appkey", tpEquipmentTree.getAppkey());
            parOpenCard.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
            parOpenCard.put("opUserUuid", tpEquipmentTree.getOpUserUuid());
            JSONObject openCardJSON = JSONObject.parseObject(HKOpenApi.openCard(parOpenCard));
            int code = (int) openCardJSON.get("errorCode");
            if (code != 0) {
                String errorMessage = String.valueOf(openCardJSON.get("errorMessage"));
                log.error("人员绑定开卡失败! {}", errorMessage);
                throw new RuntimeException(errorMessage);
            }
        }

        // 存在人脸的时候才下发人脸 卡权限/人脸权限
        if (null != taFace) {

            //=================== 下发可是对讲卡权限 =====================

            //--------- 可视对讲 ----------

            Map<String,Object> addOutDoorAuthMap = Maps.newHashMap();
            // app用户权限勿动 unitHkSetting.getPermissionId()
            addOutDoorAuthMap.put("authName", taUser.getUserName()+taUserVerify.getId());
            addOutDoorAuthMap.put("personIds", Arrays.asList(taUserHk.getHkUserId()));
            addOutDoorAuthMap.put("secret", tpEquipmentTree.getSecret());
            addOutDoorAuthMap.put("appkey", tpEquipmentTree.getAppkey());
            addOutDoorAuthMap.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
            addOutDoorAuthMap.put("opUserUuid", tpEquipmentTree.getOpUserUuid());

            // 单元门口机
            List<TpEquipment> visualEquipments = tpEquipmentMapper.selectByCommunityIdAndUnitId(taUserVerify.getCommunityId(), taUserVerify.getUnitId(), 3);
            // 可是对讲设备号
            List<String> visualList = visualEquipments.stream()
                    .map(e-> tpConfigurationMapper.selectByPrimaryKey(e.getUuidId()).getConfigurationValue())
                    .collect(Collectors.toList());

            addOutDoorAuthMap.put("longNums", visualList); // visualList
            // 可视对讲下发 权限
            String addOutDoorAuthJSONString = HKOpenApi.addOutDoorAuth(addOutDoorAuthMap);
            JSONObject addOutDoorAuthJSON = JSONObject.parseObject(addOutDoorAuthJSONString);
            int addOutDoorAuthJSONCode = (int) addOutDoorAuthJSON.get("errorCode");
            if (addOutDoorAuthJSONCode != 0 ) {
                String errorMessage = String.valueOf(addOutDoorAuthJSON.get("errorMessage"));
                log.error("可视对讲下发 权限失败! {}", errorMessage);
                throw new RuntimeException(errorMessage);
            }

            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }


            //================ 可是对讲人脸权限下发 ==============

            // 可视对讲时 指定人员人脸权限下载
            List<Integer> userIdList = new ArrayList<>();
            userIdList.add(taUserHk.getHkUserId());

            HKOpenApi.visualIntercom(tpEquipmentTree.getOpUserUuid(), 1, userIdList, visualList, tpEquipmentTree.getAppkey(), tpEquipmentTree.getSecret(), tpEquipmentTree.getHttpServer());


            // 下发门禁权限
            ihkService.addAuthoritiesAnddownloadAuthorityByPersonIds(taUserVerify, taUserHk, tpEquipmentTree);

            // 下发门禁人脸
            ihkService.downloadFaceAndFingerInfos(tpEquipmentTree, taUserVerify, taUserHk);

        }


        TaUserVerify tempVerify = taUserVerifyMapper.selectChcekNotMessage(tpEquipmentTree.getCommunityId(), taUserVerify.getId());
        if (null == tempVerify) {
            // 推送默认消息
            sendMessage(taUserVerify);
        }


    }

    /**
     * 海康权限组
     * @return
     */
    private TpUnitHkSetting getUnitHkSetting(Integer communityId, Integer unitId) {
        TpUnitHkSetting tpUnitHkSetting = tpUnitHkSettingMapper.selectByCommunityIdAndUnitId(communityId, unitId);
        if (null == tpUnitHkSetting) {
            throw new WisdomException("权限组未配置");
        }
        return tpUnitHkSetting;
    }

    public String encodeImageToBase64(URL url) throws Exception {
        //将图片文件转化为字节数组字符串，并对其进行Base64编码处理
        System.out.println("图片的路径为:" + url.toString());
        //打开链接
        HttpURLConnection conn = null;
        try {
            conn = (HttpURLConnection) url.openConnection();
        //设置请求方式为"GET"
            conn.setRequestMethod("GET");
        //超时响应时间为5秒
            conn.setConnectTimeout(5 * 1000);
        //通过输入流获取图片数据
            InputStream inStream = conn.getInputStream();
        //得到图片的二进制数据，以二进制封装得到数据，具有通用性
            ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        //创建一个Buffer字符串
            byte[] buffer = new byte[1024];
        //每次读取的字符串长度，如果为-1，代表全部读取完毕
            int len = 0;
        //使用一个输入流从buffer里把数据读取出来
            while ((len = inStream.read(buffer)) != -1) {
        //用输出流往buffer里写入数据，中间参数代表从哪个位置开始读，len代表读取的长度
                outStream.write(buffer, 0, len);
            }
        //关闭输入流
            inStream.close();
            byte[] data = outStream.toByteArray();
        //对字节数组Base64编码
            BASE64Encoder encoder = new BASE64Encoder();
            String base64 = encoder.encode(data);
            System.out.println("网络文件[{}]编码成base64字符串:[{}]"+url.toString()+base64);
            return base64;//返回Base64编码过的字节数组字符串
        } catch (IOException e) {
            e.printStackTrace();
            throw new Exception("图片上传失败,请联系客服!");
        }
    }

    @Override
    public ResponseBean cleanMessage(Integer communityId) {
        ResponseBean responseBean = new ResponseBean();

        // 查询所有没有关联消息的审核数据

         List<TaUserVerify> userVerifyList = taUserVerifyMapper.selectNotMessage(communityId);
         // TaUserVerify tempUserVerify = taUserVerifyMapper.selectByPrimaryKey(1151);
         // List<TaUserVerify> userVerifyList = Lists.newArrayList(tempUserVerify);
        for (TaUserVerify userVerify : userVerifyList) {
            TaUserVerify tempVerify = taUserVerifyMapper.selectChcekNotMessage(userVerify.getCommunityId(), userVerify.getId());
            if (null == tempVerify) {
                // 推送默认消息
                sendMessage(userVerify);
            }
        }

        return responseBean;
    }

    /**
     * 给人员推送默认消息
     * @param userVerify
     */
    private void sendMessage(TaUserVerify userVerify) {

        //审核通过后推送消息（临时数据）
        String[] messageArr= new messagePush().messagePushAll();
        Date createDate = new Date();
        for (String count:messageArr) {
            TpMessage message = new TpMessage();
            message.setCommunityId(userVerify.getCommunityId());
            message.setMessageType("9");
            message.setAdviceType("1");
            message.setModelType("1");
            message.setUuid(userVerify.getId());
            message.setUuidType("1");
            message.setSource("1");
            message.setMessageContent(count);
            message.setStatus("1");
            message.setResult("");
            message.setMeaasgeTypeId("");
            message.setReadStatus("0");
            message.setCreateUser(userVerify.getUserId());
            message.setCreateDate(createDate);
            tpMessageMapper.insert(message);
        }
    }

    @Override
    public ResponseBean cleanCardPersionHk(Integer communityId) {
        ResponseBean responseBean = new ResponseBean();

        //根据小区id查询审核数据
        List<TaUserHk> taUserHkList = taUserHkMapper.selectByCommunityId(communityId);
        for (TaUserHk taUserHk : taUserHkList) {

            TaUserVerify userVerify = taUserVerifyMapper.selectByPrimaryKey(taUserHk.getTaUserId());
            if (null == userVerify){
                continue;
            }
            TaUser taUser = taUserMapper.selectByPrimaryKey(userVerify.getUserId());

            // 海康权限信息
            TpUnitHkSetting unitHkSetting = getUnitHkSetting(userVerify.getCommunityId(), userVerify.getUnitId());
            TpEquipmentTree tpEquipmentTree = tpEquipmentTreeMapper.selectByCommunityId(userVerify.getCommunityId());

            //--------- 可视对讲 ----------

            Map<String,Object> addOutDoorAuthMap = Maps.newHashMap();
            // app用户权限勿动 unitHkSetting.getPermissionId()
            addOutDoorAuthMap.put("authName", taUser.getUserName()+userVerify.getId());
            addOutDoorAuthMap.put("personIds", Arrays.asList(taUserHk.getHkUserId()));
            addOutDoorAuthMap.put("secret", tpEquipmentTree.getSecret());
            addOutDoorAuthMap.put("appkey", tpEquipmentTree.getAppkey());
            addOutDoorAuthMap.put("openapi_ip_port_http", tpEquipmentTree.getHttpServer());
            addOutDoorAuthMap.put("opUserUuid", tpEquipmentTree.getOpUserUuid());

            // 单元门口机
            List<TpEquipment> visualEquipments = tpEquipmentMapper.selectByCommunityIdAndUnitId(userVerify.getCommunityId(), userVerify.getUnitId(), 3);
            // 可是对讲设备号
            List<String> visualList = visualEquipments.stream()
                    .map(e-> tpConfigurationMapper.selectByPrimaryKey(e.getUuidId()).getConfigurationValue())
                    .collect(Collectors.toList());

            addOutDoorAuthMap.put("longNums", visualList); // visualList
            // 可视对讲下发 权限
            String addOutDoorAuthJSONString = HKOpenApi.addOutDoorAuth(addOutDoorAuthMap);
            JSONObject addOutDoorAuthJSON = JSONObject.parseObject(addOutDoorAuthJSONString);
            int addOutDoorAuthJSONCode = (int) addOutDoorAuthJSON.get("errorCode");
            if (addOutDoorAuthJSONCode != 0 ) {
                String errorMessage = String.valueOf(addOutDoorAuthJSON.get("errorMessage"));
                log.error("可视对讲下发 权限失败! {}", errorMessage);
                throw new RuntimeException(errorMessage);
            }

            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        return responseBean;
    }
}
